;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;;; Draw a *non-filled* ractangle. The interface is compatible with
;;; draw-rectangle:
(define (draw-rectangle-border x0 y0 width height color)
  (draw-line x0 y0 (+ x0 width -1) y0 color)
  (draw-line x0 y0 x0 (+ y0 height -1) color)
  (draw-line x0 (+ y0 height -1) (+ x0 width -1) (+ y0 height -1) color)
  (draw-line (+ x0 width -1) y0 (+ x0 width -1) (+ y0 height -1) color))

;;; Draw a block with lights and shadows, suitable for Tetris and Breakout;
;;; notice that there is an optional parameter (which is seen by the procedure
;;; as an empty list when not given, and a non-empty list when given):
(define (draw-block x0 y0 width height color . border-size-or-null)
  ;; If the optional parameter was supplied then bind border-size to the
  ;; value given by the user, otherwise use a default:
  (let ((border-size (if (null? border-size-or-null)
                         3
                         (car border-size-or-null))))
    ;; Draw nothing if the block is too small:
    (if (or (<= width (* 2 border-size))
            (<= height (* 2 border-size)))
        #t ; well, we have to return something, so we return #t
        ;; Ok, the block is not too small: draw it:
        (let ((lighter-color (light color)) ; color for the lightened part
              (darker-color (dark color)))  ; color for the shadowed part
          ;; Draw the shadowed part on the East and South sides:
          (draw-rectangle x0 (+ y0 height (- border-size)) width border-size darker-color)
          (draw-rectangle (+ x0 width (- border-size)) y0 border-size height darker-color)
          ;; Draw the enlightended part on the West and Nort sides:
          (draw-rectangle x0 y0 border-size height lighter-color)
          (draw-rectangle x0 y0 width border-size lighter-color)
          ;; Draw the inside part:
          (draw-rectangle (+ x0 border-size) (+ y0 border-size)
                          (- width (* 2 border-size)) (- height (* 2 border-size))
                          color))))) ;; There's no automatic refresh here!
