/* This file is part of guile-with-graphics, a graphic library for Guile

Copyright (C) 2008, 2009  Luca Saiu
Based on turtle, by Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>
#include "sdl.h"
#include "guilesdl.h"
#include <libguile.h>
#include <guile/gh.h>
//#include <guile/readline.h>

static SCM game_color_to_red_guile(SCM color_){
  game_color_t color = game_color_smob_to_color(color_);
  return scm_from_double(color->red);
}
static SCM game_color_to_green_guile(SCM color_){
  game_color_t color = game_color_smob_to_color(color_);
  return scm_from_double(color->green);
}
static SCM game_color_to_blue_guile(SCM color_){
  game_color_t color = game_color_smob_to_color(color_);
  return scm_from_double(color->blue);
}

static SCM game_clear_window_guile(SCM color_){
  game_color_t color = game_color_smob_to_color(color_);
  game_clear_window(color);
  return SCM_UNSPECIFIED;
}

static SCM game_refresh_window_guile(void){
  game_refresh_window();
  return SCM_UNSPECIFIED;
}

static SCM game_refresh_rectangle_guile(SCM x_, SCM y_, SCM width_, SCM height_){
  const float x = scm_to_double(x_);
  const float y = scm_to_double(y_);
  const float width = scm_to_double(width_);
  const float height = scm_to_double(height_);
  game_refresh_rectangle(x, y, width, height);
  
  return SCM_UNSPECIFIED;
}

static SCM game_lock_surface_guile(void){
  game_lock_surface();
  return SCM_UNSPECIFIED;
}

static SCM game_unlock_surface_guile(void){
  game_unlock_surface();
  return SCM_UNSPECIFIED;
}

static SCM game_draw_pixel_guile(SCM x_, SCM y_, SCM color_){
  const float x = scm_to_double(x_);
  const float y = scm_to_double(y_);
  const game_color_t color = game_color_smob_to_color(color_);
  game_draw_pixel(x, y, color);
  return SCM_UNSPECIFIED;
}

static SCM game_draw_line_guile(SCM x1_, SCM y1_, SCM x2_, SCM y2_, SCM color_){
  const float x1 = scm_to_double(x1_);
  const float y1 = scm_to_double(y1_);
  const float x2 = scm_to_double(x2_);
  const float y2 = scm_to_double(y2_);
  const game_color_t color = game_color_smob_to_color(color_);
  game_draw_line(x1, y1, x2, y2, color);
  return SCM_UNSPECIFIED;
}

static SCM game_draw_rectangle_guile(SCM x0_, SCM y0_, SCM width_, SCM height_, SCM color_){
  const float x0 = scm_to_double(x0_);
  const float y0 = scm_to_double(y0_);
  const float width = scm_to_double(width_);
  const float height = scm_to_double(height_);
  const game_color_t color = game_color_smob_to_color(color_);
  game_draw_rectangle(x0, y0, width, height, color);
  return SCM_UNSPECIFIED;
}

SCM game_get_window_width_guile(void){
  return scm_from_int(game_get_window_width());
}

SCM game_get_window_height_guile(void){
  return scm_from_int(game_get_window_height());
}

SCM game_initialize_graphics_guile(SCM window_title_, SCM width_, SCM height_){
  const int width = scm_to_int(width_); 
  const int height = scm_to_int(height_);
  const char* window_title = scm_to_locale_string(window_title_);
  
  game_initialize_graphics(window_title, width, height);
  game_clear_window(game_color_black);
  game_refresh_window();

  return SCM_UNSPECIFIED;
}

SCM game_get_milliseconds_since_startup_guile(void){
  return scm_from_int((int)game_get_milliseconds_since_startup());
}

void game_initialize_guile_support(void){
  /* Initialize Guile: */
  scm_init_guile();
  
  /* Define the color SMOB, so that colors become accessible from Scheme as a
     new kind of s-expressions: */
  game_define_color_smob();

  ///* Let's make the command line more comfortable: */
  //scm_init_readline();
  
  /* Make all primitives visible from Guile: */
  scm_c_define_gsubr("initialize-graphics", 3, 0, 0, game_initialize_graphics_guile);

  scm_c_define_gsubr("clear", 1, 0, 0, game_clear_window_guile);
  scm_c_define_gsubr("clear-window", 1, 0, 0, game_clear_window_guile);

  scm_c_define_gsubr("color->red", 1, 0, 0, game_color_to_red_guile);
  scm_c_define_gsubr("color->green", 1, 0, 0, game_color_to_green_guile);
  scm_c_define_gsubr("color->blue", 1, 0, 0, game_color_to_blue_guile);

  scm_c_define_gsubr("draw-pixel", 3, 0, 0, game_draw_pixel_guile);
  scm_c_define_gsubr("draw-line", 5, 0, 0, game_draw_line_guile);
  scm_c_define_gsubr("draw-rectangle", 5, 0, 0, game_draw_rectangle_guile);

  scm_c_define_gsubr("lock", 0, 0, 0, game_lock_surface_guile);
  scm_c_define_gsubr("unlock", 0, 0, 0, game_unlock_surface_guile);
  scm_c_define_gsubr("lock-surface", 0, 0, 0, game_lock_surface_guile);
  scm_c_define_gsubr("unlock-surface", 0, 0, 0, game_unlock_surface_guile);

  scm_c_define_gsubr("refresh", 0, 0, 0, game_refresh_window_guile);
  scm_c_define_gsubr("refresh-window", 0, 0, 0, game_refresh_window_guile);
  scm_c_define_gsubr("refresh-rectangle", 4, 0, 0, game_refresh_rectangle_guile);
  scm_c_define_gsubr("redraw", 0, 0, 0, game_refresh_window_guile);
  
  scm_c_define_gsubr("get-window-width", 0, 0, 0, game_get_window_width_guile);
  scm_c_define_gsubr("window-width", 0, 0, 0, game_get_window_width_guile);
  scm_c_define_gsubr("get-window-height", 0, 0, 0, game_get_window_height_guile);
  scm_c_define_gsubr("window-height", 0, 0, 0, game_get_window_height_guile);

  scm_c_define_gsubr ("make-color", 3, 0, 0, game_make_color_guile);

  scm_c_define_gsubr ("get-event", 0, 0, 0, game_get_event_guile);

  scm_c_define_gsubr ("get-milliseconds-since-startup", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("get-milliseconds", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("get-ms-since-startup", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("get-ms", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("get-time", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("milliseconds-since-startup", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("milliseconds", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("ms-since-startup", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("ms", 0, 0, 0, game_get_milliseconds_since_startup_guile);
  scm_c_define_gsubr ("time", 0, 0, 0, game_get_milliseconds_since_startup_guile);
}
